@@ -1,7 +1,7 @@
 import os
 from collections import OrderedDict
 
-__VERSION__ = "1.14.1"
+__VERSION__ = "2.0.7"
 __version__ = __VERSION__
 
 ASSAY_LIST = [
@@ -12,26 +12,23 @@ ASSAY_LIST = [
     'snp',
     'capture_virus',
     'fusion',
-    'hla',
-    'capture_rna',
     'citeseq',
-    'flv_CR',
-    'convert10X',
     'flv_trust4',
     'sweetseq',
-    'rna_virus',
-    'utils'
+    'utils',
+    'bulk_vdj',
+    'bulk_rna',
 ]
 
 ROOT_PATH = os.path.dirname(__file__)
 
-RELEASED_ASSAYS = ['rna', 'vdj', 'tag', 'dynaseq', 'snp', 'capture_virus', 'fusion', 'citeseq', 'flv_CR', 'flv_trust4', 'sweetseq']
+RELEASED_ASSAYS = ['rna', 'vdj', 'tag', 'dynaseq', 'snp', 'capture_virus', 'fusion', 'citeseq', 'flv_trust4', 'sweetseq','bulk_vdj','bulk_rna']
 
 # argument help
 HELP_DICT = {
     'match_dir': 'Match celescope scRNA-Seq directory.',
     'gene_list': 'Required. Gene list file, one gene symbol per line. Only results of these genes are reported. Conflict with `--panel`',
-    'genomeDir': 'Required. Genome directory after running `celescope {assay} mkref`.',
+    'genomeDir': 'Genome directory after running `celescope {assay} mkref`.',
     'thread': 'Thread to use.',
     'debug': 'If this argument is used, celescope may output addtional file for debugging.',
     'fasta': 'Required. Genome fasta file. Use absolute path or relative path to `genomeDir`.',
