@@ -20,7 +20,7 @@ class Variant_calling(Step):
 
         # set
         self.barcodes, _num = utils.get_barcode_from_match_dir(args.match_dir)
-        self.fasta = Mkref_rna.parse_genomeDir(args.genomeDir)['fasta']
+        self.fasta = Mkref_rna.get_config(args.genomeDir)['files']['fasta']
         self.df_vcf = None
         self.panel = args.panel
         self.bed = utils.get_bed_file_path(self.panel)
@@ -46,15 +46,6 @@ class Variant_calling(Step):
         )
         self.debug_subprocess_call(cmd)
 
-    @utils.add_log
-    def fix_header(self):
-        cmd = (
-            'picard FixVcfHeader '
-            f'I={self.raw_vcf_file} '
-            f'O={self.fixed_header_vcf} '
-        )
-        self.debug_subprocess_call(cmd)
-
     @utils.add_log
     def gatk_norm(self):
         cmd = (
