@@ -16,17 +16,12 @@ RESOLUTION = 1.2
 N_PCS = 25
 MITO_GENE_PERCENT_LIST = [5, 10, 15, 20, 50]
 # output marker top n in html
-MARKER_TOP_N = 50
-# marker sort by
-MARKER_SORT_BY = 'p_val_adj'
+MARKER_TOP_N = 100
 
 
 def read_tsne(tsne_file):
-    df = pd.read_csv(tsne_file, sep='\t')
-    # compatible with old version
-    if 'Unnamed: 0' in df.columns:
-        df.rename(columns={'Unnamed: 0': 'barcode'}, inplace=True)
-        df = df.set_index('barcode')
+    df = pd.read_csv(tsne_file, sep='\t', index_col=0)
+    df.index.names = ['barcode']
     return df
 
 def format_df_marker(df_marker):
@@ -48,7 +43,7 @@ def get_opts_analysis(parser, sub_program):
     if sub_program:
         parser.add_argument(
             '--matrix_file',
-            help='Required. Matrix_10X directory from step count.',
+            help='Required. Matrix directory path.',
             required=True,
         )
         parser = s_common(parser)
@@ -65,13 +60,15 @@ class Scanpy_wrapper(Step):
             args.matrix_file,  
             var_names='gene_symbols',
         )
-        self.mt_gene_list = Mkref_rna.parse_genomeDir(args.genomeDir)['mt_gene_list']
+        self.adata.layers['raw'] = self.adata.X.copy()
+        self.mt_gene_list = Mkref_rna.get_config(args.genomeDir)['files']['mt_gene_list']
 
         # out
-        self.df_marker_file = f'{self.out_prefix}_markers.tsv'
-        self.df_marker_raw_file = f'{self.out_prefix}_markers_raw.tsv'
-        self.df_tsne_file = f'{self.out_prefix}_tsne_coord.tsv'
-        self.h5ad_file = f'{self.out_prefix}.h5ad'
+        self.df_marker_file = f'{self.outdir}/markers.tsv'
+        self.df_marker_raw_file = f'{self.outdir}/markers_raw.tsv'
+        self.df_tsne_file = f'{self.outdir}/tsne_coord.tsv'
+        self.h5ad_file = f'{self.outdir}/rna.h5ad'
+        self.outs = [self.df_marker_file, self.df_tsne_file, self.h5ad_file]
 
    
     @utils.add_log
@@ -207,7 +204,6 @@ class Scanpy_wrapper(Step):
         sc.tl.tsne(
             self.adata,
             n_pcs=N_PCS,
-            n_jobs=self.thread,
             copy=False,
         )
 
@@ -292,7 +288,7 @@ class Scanpy_wrapper(Step):
         df_markers = df_markers.loc[df_markers['p_val_adj'] < PVAL_CUTOFF, ]
         df_markers.to_csv(self.df_marker_raw_file, index=None, sep='\t')
 
-        df_markers_filter = df_markers.loc[df_markers['avg_log2FC'] > 0].sort_values(MARKER_SORT_BY, ascending=False).groupby('cluster').head(50)
+        df_markers_filter = df_markers.loc[df_markers['avg_log2FC'] > 0].sort_values('p_val_adj').groupby('cluster').head(100)
         df_markers_filter = df_markers_filter.round({
             'avg_log2FC':3,
             'pct.1':3,
@@ -392,7 +388,7 @@ class Report_runner(Step):
         """
         match_dict = utils.parse_match_dir(match_dir)
         df_tsne_file = match_dict['tsne_coord']
-        df_marker_file = match_dict['markers']
+        df_marker_file = match_dict.get('markers', None)
         return df_tsne_file, df_marker_file
 
     def get_df(self):
@@ -407,8 +403,11 @@ class Report_runner(Step):
         else:
             raise ValueError('match_dir or tsne_file must be specified')
         df_tsne = read_tsne(df_tsne_file)
-        df_marker = pd.read_csv(df_marker_file, sep="\t")
-        df_marker = format_df_marker(df_marker)
+        if df_marker_file:
+            df_marker = pd.read_csv(df_marker_file, sep="\t")
+            df_marker = format_df_marker(df_marker)
+        else:
+            df_marker = None
         return df_tsne, df_marker
 
     def run(self):
