@@ -1,3 +1,4 @@
+import os
 import glob
 import re
 import sys
@@ -21,7 +22,7 @@ class Chemistry():
     Auto detect chemistry from R1-read
     """
 
-    def __init__(self, fq1):
+    def __init__(self, fq1, assay=None):
         '''
         'scopeV2.0.1': 'C8L16C8L16C8L1U8T18'
         'scopeV2.1.1': 'C8L16C8L16C8L1U12T18'
@@ -29,6 +30,7 @@ class Chemistry():
         'scopeV3.0.1': 'C9L16C9L16C9L1U12T18' with 4 types of linkers
         '''
         self.fq1 = fq1
+        self.assay = assay
         self.fq1_list = fq1.split(',')
         self.n_read = 10000
 
@@ -44,9 +46,13 @@ class Chemistry():
     @utils.add_log
     def check_chemistry(self):
         """check chemistry in the fq1_list"""
+        if self.assay == 'bulk_vdj':
+            return ['bulk_vdj'] * len(self.fq1_list)
+        if self.assay == 'bulk_rna':
+            return ['bulk_rna'] * len(self.fq1_list)
         chemistry_list = []
         for fastq1 in self.fq1_list:
-            print(fastq1)
+            self.check_chemistry.logger.info(fastq1)
             chemistry = self.get_chemistry(fastq1)
             chemistry_list.append(chemistry)
         if len(set(chemistry_list)) != 1:
@@ -144,6 +150,20 @@ class Chemistry():
 
         return chemistry
 
+    @staticmethod
+    def get_whitelist(chemistry):
+        # returns: [bclists]
+        repeat = 3
+        root_dir = f'{ROOT_PATH}/data/chemistry/{chemistry}'
+        bclist = f'{root_dir}/bclist'
+        bclist1 = f'{root_dir}/bclist1'
+        if os.path.exists(bclist1):
+            return [f'{root_dir}/bclist{i}' for i in range(1,repeat+1)]
+        elif os.path.exists(bclist):
+            return [bclist] * repeat
+        else:
+            sys.exit(f'No bclist found for chemistry {chemistry} under {root_dir}')
+
 
 class Barcode(Step):
     """
@@ -171,7 +191,7 @@ class Barcode(Step):
         if self.fq_number != len(self.fq2_list):
             raise Exception('fastq1 and fastq2 do not have same file number!')
         if args.chemistry == 'auto':
-            ch = Chemistry(args.fq1)
+            ch = Chemistry(args.fq1, self.assay)
             self.chemistry_list = ch.check_chemistry()
         else:
             self.chemistry_list = [args.chemistry] * self.fq_number
@@ -196,18 +216,14 @@ class Barcode(Step):
         self.noLinker = args.noLinker
         self.output_R1 = args.output_R1
         self.bool_flv = False
+        self.wells = args.wells
+
 
-        self._assay = self.get_slot_key(
-            slot='metrics',
-            step_name='sample',
-            key='Assay',
-        )
-        self._assay = self._assay.split(' ')[-1]
         # flv_trust4, flv_CR
-        if 'flv' in self._assay:
+        if self.assay in ('flv_CR', 'flv_trust4'):
             self.bool_flv = True
             self.barcode_read_Counter = Counter()
-            if self._assay == 'flv_trust4':
+            if self.assay == 'flv_trust4':
                 if args.match_dir == 'None':
                     raise FileNotFoundError('Match_dir required when running flv_trust4')
                 self.match_barcodes = set(utils.get_barcode_from_match_dir(args.match_dir)[0]) # barcode set of flv_rna.
@@ -215,12 +231,8 @@ class Barcode(Step):
                 self.match_cbs = set() # record barcode number match with flv_rna.
 
         # out file
-        if args.gzip:
-            self.suffix = ".gz"
-        else:
-            self.suffix = ""
-        self.out_fq2 = f'{self.out_prefix}_2.fq{self.suffix}'
-        self.out_fq1 = f'{self.out_prefix}_1.fq{self.suffix}'
+        self.out_fq2 = f'{self.out_prefix}_2.fq'
+        self.out_fq1 = f'{self.out_prefix}_1.fq'
         if self.nopolyT:
             self.nopolyT_1 = f'{self.out_prefix}_noPolyT_1.fq'
             self.nopolyT_2 = f'{self.out_prefix}_noPolyT_2.fq'
@@ -246,16 +258,12 @@ class Barcode(Step):
 
         Returns:
             str
-
-        Raise:
-            IndexError: if sequence length is not enough
+            if sequence length is not enough, return ""
 
         >>> sub_pattern_dict = [[0, 8]]
         >>> seq = "A" * 7
         >>> Barcode.get_seq_str(seq, sub_pattern_dict)
-        Traceback (most recent call last):
-        ...
-        IndexError: sequence length is not enough in R1 read: AAAAAAA
+        ""
         >>> seq = "A" * 8
         >>> Barcode.get_seq_str(seq, sub_pattern_dict)
         'AAAAAAAA'
@@ -282,7 +290,6 @@ class Barcode(Step):
         return [seq[item[0]: item[1]] for item in pattern_dict[abbr]]
 
     @staticmethod
-    @utils.add_log
     def parse_pattern(pattern):
         """
         >>> pattern_dict = Barcode.parse_pattern("C8L16C8L16C8L1U12T18")
@@ -295,8 +302,7 @@ class Barcode(Step):
         p = re.compile(r'([CLUNT])(\d+)')
         tmp = p.findall(pattern)
         if not tmp:
-            Barcode.parse_pattern.logger.error(f'Invalid pattern: {pattern}')
-            sys.exit()
+            sys.exit()(f'Invalid pattern: {pattern}')
         start = 0
         for item in tmp:
             end = start + int(item[1])
@@ -321,11 +327,15 @@ class Barcode(Step):
         
 
     @staticmethod
-    def get_scope_bc(chemistry):
+    def get_scope_bc(chemistry, wells=""):
         """Return (linker file path, whitelist file path)"""
         try:
-            linker_f = glob.glob(f'{ROOT_PATH}/data/chemistry/{chemistry}/linker*')[0]
-            whitelist_f = f'{ROOT_PATH}/data/chemistry/{chemistry}/bclist'
+            if chemistry == 'bulk_rna':
+                linker_f = None
+                whitelist_f = f'{ROOT_PATH}/data/chemistry/{chemistry}/bclist{wells}'
+            else:
+                linker_f = glob.glob(f'{ROOT_PATH}/data/chemistry/{chemistry}/linker*')[0]
+                whitelist_f = f'{ROOT_PATH}/data/chemistry/{chemistry}/bclist'
         except IndexError:
             return None, None
         return linker_f, whitelist_f
@@ -370,7 +380,6 @@ class Barcode(Step):
         return seq_set
 
     @staticmethod
-    @utils.add_log
     def get_mismatch_dict(seq_list, n_mismatch=1):
         """
         Return:
@@ -402,39 +411,52 @@ class Barcode(Step):
         >>> mismatch_dict_list = [Barcode.get_mismatch_dict(['AAA'])] * 3
 
         >>> Barcode.check_seq_mismatch(seq_list, correct_set_list, mismatch_dict_list)
-        (True, True, 'AAAAAAAAA')
+        (True, True, 'AAA_AAA_AAA')
 
         >>> seq_list = ['AAA', 'AAA', 'AAA']
         >>> Barcode.check_seq_mismatch(seq_list, correct_set_list, mismatch_dict_list)
-        (True, False, 'AAAAAAAAA')
+        (True, False, 'AAA_AAA_AAA')
         '''
         bool_valid = True
         bool_corrected = False
-        corrected_seq = ''
+        corrected_seq_list = []
         for index, seq in enumerate(seq_list):
             if seq not in correct_set_list[index]:
                 if seq not in mismatch_dict_list[index]:
                     bool_valid = False
-                    return bool_valid, bool_corrected, corrected_seq
+                    return bool_valid, bool_corrected, ""
                 else:
                     bool_corrected = True
-                    corrected_seq += mismatch_dict_list[index][seq]
+                    corrected_seq_list.append(mismatch_dict_list[index][seq])
             else:
-                corrected_seq += seq
-        return bool_valid, bool_corrected, corrected_seq
+                corrected_seq_list.append(seq)
+
+        return bool_valid, bool_corrected, '_'.join(corrected_seq_list)
 
     @staticmethod
-    def parse_whitelist_file(whitelist_file, n_mismatch, n_repeat):
-        barcode_list, _ = utils.read_one_col(whitelist_file)
-        barcode_set = set(barcode_list)
-        barcode_mismatch_dict = Barcode.get_mismatch_dict(barcode_list, n_mismatch)
-        barcode_mismatch_list = [barcode_mismatch_dict] * n_repeat
-        barcode_set_list = [barcode_set] * n_repeat
-        return barcode_set_list, barcode_mismatch_list
+    def parse_whitelist_file(files: list, n_pattern: int, n_mismatch: int):
+        """
+        files: file paths
+        n_pattern: number of sections in pattern
+        n_mismatch: allowed number of mismatch bases
+        Returns:
+            white_set_list
+            mismatch_list
+        """
+        n_files = len(files)
+        if n_files == 1 and n_pattern > 1:
+            files = [files[0]] * n_pattern
+        elif n_files != n_pattern:
+            sys.exit(f'number of whitelist files({n_files} files:{files}) != n_pattern({n_pattern})')
+        
+        white_set_list, mismatch_list = [], []
+        for f in files:
+            barcodes, _ = utils.read_one_col(f)
+            white_set_list.append(set(barcodes))
+            barcode_mismatch_dict = Barcode.get_mismatch_dict(barcodes, n_mismatch)
+            mismatch_list.append(barcode_mismatch_dict)
 
-    @staticmethod
-    def parse_linker_file(linker_file):
-        return Barcode.parse_whitelist_file(linker_file, n_mismatch=2, n_repeat=1)
+        return white_set_list, mismatch_list
 
     @staticmethod
     def parse_chemistry(chemistry):
@@ -445,8 +467,8 @@ class Barcode(Step):
         pattern_dict = Barcode.parse_pattern(pattern)
         linker_file, whitelist_file = Barcode.get_scope_bc(chemistry)
 
-        barcode_set_list, barcode_mismatch_list = Barcode.parse_whitelist_file(whitelist_file, 1, len(pattern_dict['C']))
-        linker_set_list, linker_mismatch_list = Barcode.parse_linker_file(linker_file)
+        barcode_set_list, barcode_mismatch_list = Barcode.parse_whitelist_file([whitelist_file], n_pattern=len(pattern_dict['C']), n_mismatch=1)
+        linker_set_list, linker_mismatch_list = Barcode.parse_whitelist_file([linker_file],n_pattern=1, n_mismatch=2)
 
         return pattern_dict, barcode_set_list, barcode_mismatch_list, linker_set_list, linker_mismatch_list
 
@@ -465,7 +487,8 @@ class Barcode(Step):
     def open_files(self):
         if self.output_R1 or self.bool_flv:
             self.fh_fq1 = xopen(self.out_fq1, 'w')
-        self.fh_fq2 = xopen(self.out_fq2, 'w')
+        if not self.args.stdout:
+            self.fh_fq2 = xopen(self.out_fq2, 'w')
 
         if self.nopolyT:
             self.fh_nopolyT_fq1 = xopen(self.nopolyT_1, 'w')
@@ -478,7 +501,8 @@ class Barcode(Step):
     def close_files(self):
         if self.output_R1 or self.bool_flv:
             self.fh_fq1.close()
-        self.fh_fq2.close()
+        if not self.args.stdout:
+            self.fh_fq2.close()
 
         if self.nopolyT:
             self.fh_nopolyT_fq1.close()
@@ -570,7 +594,7 @@ class Barcode(Step):
         if self.clean_num == 0:
             raise Exception('no valid reads found! please check the --chemistry parameter.' + HELP_DICT['chemistry'])
         
-        if self._assay == 'flv_trust4':
+        if self.assay == 'flv_trust4':
             self.add_metric(
                 name='Valid Matched Reads',
                 value=self.match_num,
@@ -614,11 +638,13 @@ class Barcode(Step):
             # get linker and whitelist
             bc_pattern = PATTERN_DICT[chemistry]
             if (bc_pattern):
-                linker_file, whitelist_file = self.get_scope_bc(chemistry)
+                linker_file, whitelist_file = self.get_scope_bc(chemistry, self.wells)
+                whitelist_files = [whitelist_file]
             else:
                 bc_pattern = self.pattern
                 linker_file = self.linker
                 whitelist_file = self.whitelist
+                whitelist_files = whitelist_file.split(',')
             if not bc_pattern:
                 raise Exception("invalid bc_pattern!")
 
@@ -630,10 +656,10 @@ class Barcode(Step):
             C_len = sum([item[1] - item[0] for item in pattern_dict['C']])
 
             if bool_whitelist:
-                barcode_set_list, barcode_mismatch_list = Barcode.parse_whitelist_file(whitelist_file,
-                                                                               n_mismatch=1, n_repeat=len(pattern_dict['C']))
+                barcode_set_list, barcode_mismatch_list = Barcode.parse_whitelist_file(whitelist_files,
+                                                n_pattern=len(pattern_dict['C']), n_mismatch=1)
             if bool_L:
-                linker_set_list, linker_mismatch_list = Barcode.parse_linker_file(linker_file)
+                linker_set_list, linker_mismatch_list = Barcode.parse_whitelist_file([linker_file],n_pattern=1, n_mismatch=2)
 
             with pysam.FastxFile(self.fq1_list[i], persist=False) as fq1, \
                     pysam.FastxFile(self.fq2_list[i], persist=False) as fq2:
@@ -668,9 +694,9 @@ class Barcode(Step):
                         if not bool_valid:
                             self.no_linker_num += 1
                             if self.noLinker:
-                                self.fh_noLinker_fq1(
-                                    '@%s\n%s\n+\n%s\n' % (header1, seq1, qual1))
-                                self.fh_noLinker_fq2(
+                                self.fh_nolinker_fq1.write(
+                                    f'@{header1}\n{seq1}\n{seq_str}\n{qual1}\n')
+                                self.fh_nolinker_fq2.write(
                                     '@%s\n%s\n+\n%s\n' % (header2, seq2, qual2))
                             continue
                         elif bool_corrected:
@@ -691,31 +717,32 @@ class Barcode(Step):
                             self.barcode_corrected_num += 1
                         cb = corrected_seq
                     else:
-                        cb = "".join(seq_list)
+                        cb = "_".join(seq_list)
 
                     self.clean_num += 1
                     self.barcode_qual_Counter.update(C_U_quals_ascii[:C_len])
                     self.umi_qual_Counter.update(C_U_quals_ascii[C_len:])
 
                     umi = Barcode.get_seq_str(seq1, pattern_dict['U'])
+                    if not umi:
+                        continue
 
                     if self.bool_flv:
                         qual1 = 'F' * len(cb + umi)
                         self.barcode_read_Counter.update(cb)
-                        if self._assay == 'flv_trust4' and cb in self.match_barcodes:
+                        if self.assay == 'flv_trust4' and cb in self.match_barcodes:
                             self.match_num += 1
                             self.match_cbs.add(cb)
                             if self.barcode_read_Counter[cb] <= 80000:
-                                self.fh_fq2.write(f'@{cb}_{umi}_{self.total_num}\n{seq2}\n+\n{qual2}\n')
-                                self.fh_fq1.write(f'@{cb}_{umi}_{self.total_num}\n{cb}{umi}\n+\n{qual1}\n')
-                        elif self._assay == 'flv_CR':
-                            self.fh_fq2.write(f'@{cb}_{umi}_{self.total_num}\n{seq2}\n+\n{qual2}\n')
-                            self.fh_fq1.write(f'@{cb}_{umi}_{self.total_num}\n{cb}{umi}\n+\n{qual1}\n')
-
+                                self.fh_fq2.write(f'@{cb}:{umi}:{self.total_num}\n{seq2}\n+\n{qual2}\n')
+                                self.fh_fq1.write(f'@{cb}:{umi}:{self.total_num}\n{cb}{umi}\n+\n{qual1}\n')
                     else:
-                        self.fh_fq2.write(f'@{cb}_{umi}_{self.total_num}\n{seq2}\n+\n{qual2}\n')
+                        if self.args.stdout:
+                            print(f'@{cb}:{umi}:{self.total_num}\n{seq2}\n+\n{qual2}')
+                        else:
+                            self.fh_fq2.write(f'@{cb}:{umi}:{self.total_num}\n{seq2}\n+\n{qual2}\n')
                         if self.output_R1:
-                            self.fh_fq1.write(f'@{cb}_{umi}_{self.total_num}\n{seq1}\n+\n{qual1}\n')                   
+                            self.fh_fq1.write(f'@{cb}:{umi}:{self.total_num}\n{seq1}\n+\n{qual1}\n')                   
             
                 self.run.logger.info(self.fq1_list[i] + ' finished.')
 
@@ -786,20 +813,26 @@ lowQual will be regarded as low-quality bases.',
         help="Allow valid reads without correct linker.",
         action='store_true'
     )
-    parser.add_argument(
-        '--gzip',
-        help="Output gzipped fastq files.",
-        action='store_true'
-    )
     parser.add_argument(
         '--output_R1',
         help="Output valid R1 reads.",
         action='store_true'
     )
+    parser.add_argument(
+        '--wells',
+        help='The AccuraCode wells used (384 or 96).',
+        type=int,
+        default=384
+    )
     if sub_program:
         parser.add_argument('--fq1', help='R1 fastq file. Multiple files are separated by comma.', required=True)
         parser.add_argument('--fq2', help='R2 fastq file. Multiple files are separated by comma.', required=True)
         parser.add_argument('--match_dir', help='Matched scRNA-seq directory, required for flv_trust4')
+        parser.add_argument(
+            '--stdout',
+            help="Write output to standard output",
+            action='store_true'
+    )
         parser = s_common(parser)
 
     return parser
