@@ -1,4 +1,4 @@
-__STEPS__ = ["mkref", "sample", "barcode", "cutadapt", "consensus", "mapping_vdj", "count_vdj"]
+STEPS = ["mkref", "sample", "barcode", "cutadapt", "consensus", "mapping_vdj", "count_vdj"]
 __ASSAY__ = 'vdj'
 
 CHAINS = {
